#include "pff.h"
#include "rtc.h"
#include "usart.h"
#include "dht22.h"
#include "ssd1306.h"
#include "i2c_master.h"

#include <time.h>
#include <avr/io.h>
#include <string.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define SOL 0
#define LUMINA 2
#define APA 1
#define NO_WATER 50
#define WATER_SOIL 600

uint8_t display_buffer[1024];

FATFS fs;	// sistemul de fisiere

void show_info(rtc_t rtc, uint16_t lumina, uint16_t temp, uint16_t humidity) {
	char buffer[256];

	reset_display();	// Clear screen
	setXY(0, 1);
	sprintf(buffer, "%d:%d:%d", rtc.hour, rtc.min, rtc.sec);
	sendStr(buffer);

	setXY(2, 2);
	sprintf(buffer, "Temp: %d C", temp);
	sendStr(buffer);

	setXY(4, 2);
	sprintf(buffer, "Humidity: %d%%", humidity);
	sendStr(buffer);

	setXY(6, 2);
	sprintf(buffer, "Light: %d", lumina);
	sendStr(buffer);
}

void show_noWater() {
	reset_display();	// Clear screen

	setXY(4, 4);
	sendStr("NO WATER");
}

/* Tensiunea de referinta utilizata de ADC. */
#define ADC_AREF_VOLTAGE 5
#define ADC_MAX_CHANNELS 6

/* Numele canalelor de ADC. */
typedef struct {
  const char *name;
  uint8_t    channel;
} adc_channel_t;

/* Perechi de (Nume canal, Numar canal). */
const adc_channel_t ADC_channels[] = {
	{"Umiditate_sol", 0},
	{"Senzor_apa", 1},
	{"Lumina", 3},
};

/* Numarul de canale ADC utilizate. */
const int adc_num_channels = sizeof(ADC_channels) / sizeof(ADC_channels[0]);

/* Numarul canalului initial. */
#define ADC_INIT_CHANNEL 0

/* Valorile citite de la fiecare canal. */
volatile uint16_t ADC_value[ADC_MAX_CHANNELS];
volatile int sec;
volatile int secunde;

ISR(ADC_vect) {
    // Intreruperea este triggered cand o conversie este finalizata. */
    static uint8_t channel = ADC_INIT_CHANNEL;

    ADC_value[channel] = ADC;
    channel = (channel + 1) % ADC_MAX_CHANNELS;

	if (channel == 2) {
        channel = 3;
    }

    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    // Pornim ADC-ul daca le-am citit pe toate.
    if (channel != ADC_INIT_CHANNEL)
		ADCSRA |= (1 << ADSC);
}

ISR(TIMER1_COMPA_vect) {
	sec++;
	if(sec == 10) {
		sec = 0;
		secunde ++;
	}
    ADCSRA |= (1 << ADSC);
}

/*
 * Functia initializeaza convertorul Analog-Digital.
 */
void ADC_init(void) {
    DDRA = 0x0;   
    ADMUX = (1 << REFS0) | (1 << ADC_INIT_CHANNEL);
    ADCSRA = (1 << ADEN) | (1 << ADIE) | (5 << ADPS0);
}

/*
 * Functia porneste o noua conversie pentru canalul precizat.
 * In modul fara intreruperi, apelul functiei este blocant. Aceasta se
 * intoarce cand conversia este finalizata.
 *
 * In modul cu intreruperi apelul NU este blocant.
 *
 * @return Valoarea numerica raw citita de ADC de pe canlul specificat.
 */
uint16_t ADC_get(uint8_t channel) {
    return ADC_value[channel];
}

/*
 * Functia primeste o valoare numerica raw citita de convertul Analog-Digital
 * si calculeaza tensiunea (in volti) pe baza tensiunei de referinta.
 */
double ADC_voltage(int raw) {
    return raw * ADC_AREF_VOLTAGE / 1023.0;
}

/*
 * Functia initializeaza Timer1 pentru a genera o intrerupere care sa
 * porneasca ADC-ul la fiecare 100ms.
 */
void TIMER1_init() {
    // initialize timer to trigger COMPA every 100ms
    TCCR1B = (5 << CS10) | (1 << WGM12);
    TIMSK1 |= (1 << OCIE1A);
    OCR1A = 1561;
	sec = 0;
	secunde = 0;
}

int getSec() {
	return secunde;
}

void resetSec() {
	secunde = 0;
}

void print_in_sdCard(rtc_t rtc) {
	FRESULT ret;
    WORD bytesWritten;

	char buffer[256];

	if(pf_mount(&fs) != FR_OK) {
		USART0_print("nu a montat\n");
		_delay_ms(1000);
		return;
	}

	if((ret = pf_open("history.txt")) == FR_OK) {
		sprintf(buffer, "Planta a fost udata la %d:%d:%d\n\n", rtc.hour, rtc.min, rtc.sec);

		pf_write(buffer, 1, &bytesWritten);
	}
}

int main() {
	char buffer[256];
    rtc_t rtc;

    I2C_init();
    RTC_init();
    ADC_init();
    USART0_init();

	/* Setez pinul PD7(LED-red) ca iesire. */
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);

	/* Setez pinul PB1(pompa) ca iesire. */
	DDRB |= (1 << PB1);
	PORTB &= ~(1 << PB1);

	/* Setez pinul PC7(LED-green) ca iesire. */
	DDRC |= (1 << PC7);
	PORTC &= ~(1 << PC7);

    // init timer
    TIMER1_init();

    // Enable interrupts.
    sei();

    memset(display_buffer, 0X04, 1024);

    int tempC = 0;
    int humidity = 0;

	int pomp_started = -1;
	int started = -1;
	int flag = 0;

    InitializeDisplay();

	_delay_ms(1000);

    while(1) {
		RTC_getDateTime(&rtc);

		dht22Init();
        humidity = dht22Humidity();
        tempC = dht22Temp();

		if (ADC_get(ADC_channels[APA].channel) < NO_WATER) {
			PORTB &= ~(1 << PB1);
			PORTC &= ~(1 << PC7);
			PORTD |= (1 << PD7);
			_delay_ms(1000);
			show_noWater();
			_delay_ms(1000);
			show_info(rtc, ADC_get(ADC_channels[LUMINA].channel), (uint16_t) tempC / 10, (uint16_t) humidity / 10);
			continue;
		}

		PORTD &= ~(1 << PD7);

		if ((secunde - started) > 20 && flag == 1) {
			USART0_print("Am asteptat destul, acum planta poate fi udata iar daca este uscata\n");
			flag = 0;
		}

		if ((PINB & (1 << PB2)) == 0) {
			PORTB |= (1 << PB1);
			PORTC |= (1 << PC7);
		} else {
			PORTB &= ~(1 << PB1);
			PORTC &= ~(1 << PC7);
		}

		if (ADC_get(ADC_channels[SOL].channel) > WATER_SOIL && flag == 0) {
			PORTB |= (1 << PB1);
			PORTC |= (1 << PC7);

			if (pomp_started == -1) {
				USART0_print("Incep sa ud planta\n");

				print_in_sdCard(rtc);
				pomp_started = secunde;
				started = secunde;
			}

			if (secunde - pomp_started > 2) {
				USART0_print("Am terminat de udat planta\n");
				pomp_started = -1;
				flag = 1;
			}
		}

        char buf[32];

		for (int i = 0; i < adc_num_channels; i++) {
			const adc_channel_t *channel = &ADC_channels[i];

			sprintf(buf, "%s: %d\n", channel->name, ADC_get(channel->channel));

			USART0_print(buf);
		}

		show_info(rtc, ADC_get(ADC_channels[LUMINA].channel), (uint16_t) tempC / 10, (uint16_t) humidity / 10);

        _delay_ms(1000);
    }

	return 0;
}
